#!/usr/bin/env node

'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var argueCli = require('argue-cli');
var chalk = _interopDefault(require('chalk'));
var Table = _interopDefault(require('easy-table'));
var src = require('./');

const {
  help,
  verbose,
  ...regExpOptions
} = argueCli.options([['help', 'h'], ['verbose', 'v'], 'ignoreMajor', 'ignorePatch', 'allowHigherVersions', 'allowZeroVersions'], []);

if (help) {
  argueCli.end();
  const optionsTable = new Table();
  optionsTable.cell('Option', 'query');
  optionsTable.cell('Description', 'Manually provide a browserslist query.' + ' Specifying this overrides the browserslist configuration specified in your project.');
  optionsTable.newRow();
  optionsTable.cell('Option', '--help, -h');
  optionsTable.cell('Description', 'Print this message.');
  optionsTable.newRow();
  optionsTable.cell('Option', '--verbose, -v');
  optionsTable.cell('Description', 'Print additional info about RegExps.');
  optionsTable.newRow();
  optionsTable.cell('Option', '--ignorePatch');
  optionsTable.cell('Description', 'Ignore differences in patch browser numbers.');
  optionsTable.cell('Default', 'true');
  optionsTable.newRow();
  optionsTable.cell('Option', '--ignoreMinor');
  optionsTable.cell('Description', 'Ignore differences in minor browser versions.');
  optionsTable.cell('Default', 'false');
  optionsTable.newRow();
  optionsTable.cell('Option', '--allowHigherVersions');
  optionsTable.cell('Description', 'For all the browsers in the browserslist query,' + ' return a match if the user agent version is equal to or higher than the one specified in browserslist.');
  optionsTable.cell('Default', 'false');
  optionsTable.newRow();
  optionsTable.cell('Option', '--allowZeroSubverions');
  optionsTable.cell('Description', 'Ignore match of patch or patch and minor, if they are 0.');
  optionsTable.cell('Default', 'false');
  optionsTable.newRow();
  console.log(`\nbrowserslist-useragent-regexp [query] [...options]\n\n${optionsTable.toString()}`);
  process.exit(0);
}

const query = argueCli.argv.length ? argueCli.read() : undefined;
const options = {
  browsers: query,
  ...src.defaultOptions,
  ...regExpOptions
};
argueCli.end();

if (verbose) {
  const browsersList = src.getBrowsersList(options);
  const mergedBrowsers = src.mergeBrowserVersions(browsersList);
  console.log(chalk.blue('\n> Browserslist\n'));
  const browsersTable = new Table();
  mergedBrowsers.forEach((versions, browser) => {
    browsersTable.cell('Browser', chalk.yellow(browser));
    versions.forEach((version, i) => {
      if (src.isAllVersion(version)) {
        browsersTable.cell(`Version ${i}`, version[0]);
      } else {
        browsersTable.cell(`Version ${i}`, version.join('.'));
      }
    });
    browsersTable.newRow();
  });
  console.log(browsersTable.print());
  const rangedBrowsers = src.browserVersionsToRanges(mergedBrowsers);
  const sourceRegExps = src.getRegExpsForBrowsers(mergedBrowsers, options);
  const versionedRegExps = src.applyVersionsToRegExps(sourceRegExps, rangedBrowsers, options);
  const patchedRegExps = src.patchRegExps(versionedRegExps, mergedBrowsers);
  const optimizedRegExps = src.optimizeAll(patchedRegExps);
  console.log(chalk.blue('\n> RegExps\n'));
  optimizedRegExps.forEach(({
    family,
    requestVersionsStrings,
    sourceRegExp,
    resultFixedVersion,
    resultMinVersion,
    resultMaxVersion,
    regExp
  }) => {
    const regExpsTable = new Table();
    regExpsTable.cell('Name', chalk.yellow('Family:'));
    regExpsTable.cell('Value', family);
    regExpsTable.newRow();
    regExpsTable.cell('Name', chalk.yellow('Versions:'));
    regExpsTable.cell('Value', requestVersionsStrings.join(' '));
    regExpsTable.newRow();
    regExpsTable.cell('Name', chalk.yellow('Source RegExp:'));
    regExpsTable.cell('Value', sourceRegExp);
    regExpsTable.newRow();
    regExpsTable.cell('Name', chalk.yellow('Source RegExp fixed version:'));
    regExpsTable.cell('Value', resultFixedVersion ? resultFixedVersion.join('.') : '...');
    regExpsTable.newRow();
    let regExpBrowsersVersion = '';

    if (resultMinVersion) {
      regExpBrowsersVersion = resultMinVersion.join('.');
    } else {
      regExpBrowsersVersion = '...';
    }

    regExpBrowsersVersion += ' - ';

    if (resultMaxVersion) {
      regExpBrowsersVersion += resultMaxVersion.join('.');
    } else {
      regExpBrowsersVersion += '...';
    }

    regExpsTable.cell('Name', chalk.yellow('Source RegExp browsers versions:'));
    regExpsTable.cell('Value', regExpBrowsersVersion);
    regExpsTable.newRow();
    regExpsTable.cell('Name', chalk.yellow('Versioned RegExp:'));
    regExpsTable.cell('Value', regExp);
    regExpsTable.newRow();
    console.log(`${regExpsTable.print()}\n`);
  });
  const regExpStr = src.joinVersionedBrowsersRegExps(optimizedRegExps);
  const regExp = new RegExp(regExpStr);
  console.log(regExp);
  process.exit(0);
}

console.log(src.getUserAgentRegExp(options));
//# sourceMappingURL=data:application/json;charset=utf-8;base64,eyJ2ZXJzaW9uIjozLCJmaWxlIjoiY2xpLmpzIiwic291cmNlcyI6W10sInNvdXJjZXNDb250ZW50IjpbXSwibmFtZXMiOltdLCJtYXBwaW5ncyI6Ijs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7Ozs7OyJ9
